@@ -14,7 +14,63 @@ class Command(object):
         self.enabled = enabled
 
     def __str__(self):
-        return "{ \"%s\", %s }," % (self.name, self.handler)
+        return "{ genericName: \"%s\", handler: %s }," % (self.name, self.handler)
+
+
+class DiagnosticMessage(object):
+    def __init__(self, message_set, enabled=True, **kwargs):
+        self.message_set = message_set
+        self.enabled = enabled
+        self.id = kwargs['id']
+        self.bus = kwargs['bus']
+        self.name = kwargs.get('name', None)
+        self.multiple_responses = kwargs.get('multiple_responses', False)
+        self.factor = kwargs.get('factor', None)
+        self.offset = kwargs.get('offset', None)
+        self.decoder = kwargs.get('decoder', None)
+        self.callback = kwargs.get('callback', None)
+        self.mode = kwargs['mode']
+        self.pid = kwargs.get('pid', 0)
+        self.pid_length = kwargs.get('pid_length', 0)
+
+        try:
+            self.frequency = kwargs['frequency']
+        except KeyError:
+            raise ConfigurationError("Frequency must be specified for "
+                "pre-configured diagnostic request (bus: "
+                "%s, id: %d, mode: %d, pid: %d" % (self.bus, self.id, self.mode, self.pid))
+
+    def __str__(self):
+        result = "{\n        DiagnosticRequest request = {arbitration_id: 0x%x, mode: 0x%x, has_pid: %s, pid: 0x%x, pid_length: %d};\n" % (
+                self.id, self.mode, str(self.pid > 0).lower(), self.pid,
+                self.pid_length)
+
+        if self.name:
+            name = "\"%s\"" % self.name
+        else:
+            name = "NULL"
+
+        if self.factor is not None:
+            factor = self.factor
+        else:
+            factor = 1.0
+
+        if self.offset is not None:
+            offset = self.offset
+        else:
+            offset = 0
+
+
+        result += "        addRecurringRequest(diagnosticsManager, &getCanBuses()[%d], &request, %s, %s, %f, %f, %s, %s, %f);\n        }\n" % (
+                self.message_set.lookup_bus_index(self.bus),
+                name,
+                str(self.multiple_responses).lower(),
+                factor,
+                offset,
+                self.decoder or "NULL",
+                self.callback or "NULL",
+                self.frequency)
+        return result
 
 
 class Message(object):
@@ -34,8 +90,7 @@ class Message(object):
 
     @property
     def active(self):
-        return self.enabled and (len(list(self.enabled_signals())) > 0
-                or len(self.handlers) > 0)
+        return self.enabled
 
     @property
     def id(self):
@@ -147,8 +202,12 @@ class Message(object):
     def __str__(self):
         bus_index = self.message_set.lookup_bus_index(self.bus_name)
         if bus_index is not None:
-            return "{&CAN_BUSES[%d][%d], 0x%x, {%d}, %s}, // %s" % (
+            id_format = "STANDARD";
+            if self.id > 2047:
+                id_format = "EXTENDED";
+            return "{ bus: &CAN_BUSES[%d][%d], id: 0x%x, format: %s, frequencyClock: {%f}, forceSendChanged: %s}, // %s" % (
                     self.message_set.index, bus_index, self.id,
+                    id_format,
                     self.max_frequency,
                     str(self.force_send_changed).lower(),
                     self.name)
@@ -211,10 +270,10 @@ class CanBus(object):
         self.messages.append(message)
 
     def __str__(self):
-        result = """        {{ {bus_speed}, {controller}, can{controller},
-            {max_message_frequency}, {raw_writable},
+        result = """        {{ speed: {bus_speed}, address: {controller}, controller: can{controller},
+            maxMessageFrequency: {max_message_frequency}, rawWritable: {raw_writable},
             #ifdef __PIC32__
-            handleCan{controller}Interrupt,
+            interruptHandler: handleCan{controller}Interrupt,
             #endif // __PIC32__
         }},"""
         return result.format(bus_speed=self.speed, controller=self.controller,
@@ -222,6 +281,9 @@ class CanBus(object):
                 raw_writable=str(self.raw_writable).lower())
 
 
+class ConfigurationError(Exception):
+    pass
+
 class BitInversionError(Exception):
     pass
 
@@ -235,8 +297,8 @@ class Signal(object):
         self.generic_name = None
         self.bit_position = None
         self.bit_size = None
-        self.handler = None
-        self.write_handler = None
+        self.decoder = None
+        self.encoder = None
         self.factor = 1
         self.offset = 0
         self.min_value = 0.0
@@ -265,9 +327,9 @@ class Signal(object):
         # Kind of nasty, but we want to avoid actually setting one of the
         # implicit handlers on the object (and then later on, assuming that it
         # was set explicitly)
-        self.handler = data.get('handler', getattr(self, '_handler', None))
+        self.decoder = data.get('decoder', getattr(self, '_decoder', None))
         self.writable = data.get('writable', self.writable)
-        self.write_handler = data.get('write_handler', self.write_handler)
+        self.encoder = data.get('encoder', self.encoder)
         self.send_same = data.get('send_same', self.send_same)
         self.force_send_changed = data.get('force_send_changed',
                 self.force_send_changed)
@@ -281,18 +343,18 @@ class Signal(object):
                     " - see the replacement, max_frequency")
 
     @property
-    def handler(self):
-        handler = getattr(self, '_handler', None)
-        if handler is None:
+    def decoder(self):
+        decoder = getattr(self, '_decoder', None)
+        if decoder is None:
             if self.ignore:
-                handler = "ignoreHandler"
+                decoder = "ignoreDecoder"
             elif len(self.states) > 0:
-                handler = "stateHandler"
-        return handler
+                decoder = "stateDecoder"
+        return decoder
 
-    @handler.setter
-    def handler(self, value):
-        self._handler = value
+    @decoder.setter
+    def decoder(self, value):
+        self._decoder = value
 
     @property
     def enabled(self):
@@ -397,8 +459,7 @@ class Signal(object):
         return inverted_index
 
     def __str__(self):
-        result =  ("{&CAN_MESSAGES[%d][%d], \"%s\", %s, %d, %f, %f, %f, %f, "
-                    "{%d}, %s, %s, " % (
+        result =  ("{message: &CAN_MESSAGES[%d][%d], genericName: \"%s\", bitPosition: %s, bitSize: %d, factor: %f, offset: %f, minValue: %f, maxValue: %f, frequencyClock: {%f}, sendSame: %s, forceSendChanged: %s, " % (
                 self.message_set.index,
                 self.message_set.lookup_message_index(self.message),
                 self.generic_name, self.bit_position, self.bit_size,
@@ -406,12 +467,13 @@ class Signal(object):
                 self.max_frequency, str(self.send_same).lower(),
                 str(self.force_send_changed).lower()))
         if len(self.states) > 0:
-            result += "SIGNAL_STATES[%d][%d], %d" % (self.message_set.index,
+            result += "states: SIGNAL_STATES[%d][%d], stateCount: %d" % (self.message_set.index,
                     self.states_index, len(self.states))
         else:
-            result += "NULL, 0"
-        result += ", %s, %s" % (str(self.writable).lower(),
-                self.write_handler or "NULL")
+            result += "states: NULL, stateCount: 0"
+        result += ", writable: %s, decoder: %s, encoder: %s" % (str(self.writable).lower(),
+                self.decoder or "NULL",
+                self.encoder or "NULL")
         result += "}, // %s" % self.name
         return result
 
@@ -422,4 +484,4 @@ class SignalState(object):
         self.name = name
 
     def __str__(self):
-        return "{%d, \"%s\"}" % (self.value, self.name)
+        return "{value: %d, name: \"%s\"}" % (self.value, self.name)
